"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.calculateValidImageDimensions = calculateValidImageDimensions;
exports.resizeImage = resizeImage;
exports.shouldResizeImage = shouldResizeImage;
exports.toFile = toFile;
exports.validateImageDimensions = validateImageDimensions;

var _errors = require("./errors");

function areImageDimensionsInvalid(image, width, height) {
  return width && image.width !== width || height && image.height !== height;
}

function isImageTooSmall(image, minWidth, minHeight) {
  return image.width < minWidth || image.height < minHeight;
}

function isImageTooBig(image, maxWidth, maxHeight) {
  return image.width > maxWidth || image.height > maxHeight;
}

function validateWithAutoResize(image, options) {
  var width = options.width,
      height = options.height,
      minWidth = options.minWidth,
      minHeight = options.minHeight;

  if (minWidth && minHeight) {
    return isImageTooSmall(image, minWidth, minHeight) ? (0, _errors.imageDimensionsTooSmallError)(minWidth, minHeight) : null;
  }

  if (width && height) {
    return isImageTooSmall(image, width, height) ? (0, _errors.imageDimensionsTooSmallError)(width, height) : null;
  }

  return null;
}

function validateExactDimensions(image, options) {
  var width = options.width,
      height = options.height,
      maxWidth = options.maxWidth,
      maxHeight = options.maxHeight,
      minWidth = options.minWidth,
      minHeight = options.minHeight;

  if (areImageDimensionsInvalid(image, width, height)) {
    return (0, _errors.imageDimensionsInvalidError)(width, height);
  }

  if (isImageTooSmall(image, minWidth, minHeight)) {
    return (0, _errors.imageDimensionsTooSmallError)(minWidth, minHeight);
  }

  if (isImageTooBig(image, maxWidth, maxHeight)) {
    return (0, _errors.imageDimensionsTooBigError)(maxWidth, maxHeight);
  }

  return null;
}

function validateImageDimensions(image, options) {
  var autoResize = options.autoResize;

  if (autoResize) {
    return validateWithAutoResize(image, options);
  }

  return validateExactDimensions(image, options);
}

function shouldResizeImage(image, options) {
  var autoResize = options.autoResize,
      width = options.width,
      height = options.height,
      maxWidth = options.maxWidth,
      maxHeight = options.maxHeight;

  if (!autoResize) {
    return false;
  }

  if (width && height) {
    return areImageDimensionsInvalid(image, width, height);
  }

  if (maxHeight && maxWidth) {
    return isImageTooBig(image, maxWidth, maxHeight);
  }

  return false;
}

function toFile(base64String, name, type) {
  var byteCharacters = atob(base64String);
  var byteNumbers = new Array(byteCharacters.length);

  for (var i = 0; i < byteCharacters.length; i++) {
    byteNumbers[i] = byteCharacters.charCodeAt(i);
  }

  var byteArray = new Uint8Array(byteNumbers);
  return new File([byteArray], name, {
    type: type
  });
}
/**
 * Resize provided image and return it as a Blob object
 * @param image - image to be resized
 * @param width - future image width
 * @param height - future image height
 * @param type - resized image type - supports common image types ('image/png', 'image/jpeg', ..)
 */


function resizeImage(image, width, height, name) {
  var type = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 'image/png';
  var canvas = document.createElement('canvas');
  canvas.width = width;
  canvas.height = height;
  var ctx = canvas.getContext('2d');
  ctx.drawImage(image, 0, 0, width, height);
  var base64Image = canvas.toDataURL(type);
  var formattedBase64Image = base64Image.split(',')[1];
  return toFile(formattedBase64Image, name, type);
}
/**
 * Calculates dimensions that image should have to satisfy given restraints.
 * Currently, only image downsizing is supported so dimensions are calculated only
 * if image is larger than allowed.
 * Both width and height or both maxWidth and maxHeight must be provided through options.
 *
 * If width and height are provided, those exact dimensions will be returned.
 * If maxWidth and maxHeight are provided, max possible dimensions will be calculated,
 * keeping image aspect ratio intact.
 *
 * @param image   - original image
 * @param options - must have either width & height or maxWidth & maxHeight
 * @returns object with width and height properties
 */


function calculateValidImageDimensions(image, options) {
  var width = options.width,
      height = options.height,
      maxWidth = options.maxWidth,
      maxHeight = options.maxHeight;

  if (width && height) {
    return {
      width: width,
      height: height
    };
  }

  if (!(maxWidth && maxHeight)) {
    return null;
  }

  var imageWidth = image.width,
      imageHeight = image.height;

  if (imageWidth <= 0 || imageHeight <= 0) {
    return null;
  }

  if (imageWidth <= maxWidth && imageHeight <= maxHeight) {
    return {
      width: imageWidth,
      height: imageHeight
    };
  }

  var ratio = Math.min(maxWidth / imageWidth, maxHeight / imageHeight);
  return {
    width: Math.floor(imageWidth * ratio),
    height: Math.floor(imageHeight * ratio)
  };
}